#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'find_line_in_file/class_methods.rb'
# FindLineInFile.find[search_term: 'foo', where: '/this/file.rb']
# =========================================================================== #
class FindLineInFile

  # ========================================================================= #
  # === FindLineInFile.find
  #
  # The first argument to this method should be the search string in
  # question, that is, the line you want to find exactly.
  #
  # The second argument should ideally be a Hash but it can also be a
  # String, hence the check below in the method body. It will tell
  # us the file location.
  #
  # If the line has been found then this method will return an Integer
  # number, aka the fileline - starting at line number 1. (There is,
  # logically, no line number called 0, hence why the code here behaves
  # in that way.)
  #
  # Usage examples:
  #
  #   FindLineInFile.find(search_term, :in => 'test.txt')
  #   FindLineInFile[search_term: 'foo', where: '/this/file.rb']
  #
  # ========================================================================= #
  def self.find(
      search_term, where = ''
    )
    if search_term.is_a? Hash
      # ===================================================================== #
      # === :where
      # ===================================================================== #
      if search_term.has_key? :where
        if where.is_a?(String) and where.empty?
          where = search_term.delete(:where)
        end
      end
      # ===================================================================== #
      # === :what
      # ===================================================================== #
      if search_term.has_key? :what
        if search_term.has_key? :in
          where = search_term.delete :in
        elsif search_term.has_key? :where
          where = search_term.delete :where
        end
        search_term = search_term.delete :what
      end
      # ===================================================================== #
      # === :search_term
      #
      # This should come last.
      # ===================================================================== #
      if search_term.has_key? :search_term
        search_term = search_term.delete(:search_term)
      end
    end
    if where.is_a? Hash
      if where.has_key? :in
        where = where.delete :in
      elsif where.has_key? :where
        where = where.delete :where
      end
    elsif where.is_a? String # Handle given Strings next.
      where = where.to_s.dup
    end
    # ======================================================================= #
    # `where` may still be a Hash at this point, so
    # we convert it into a string.
    # ======================================================================= #
    if where.is_a? Hash and where.empty?
      where = ''
    end
    _ = FindLineInFile.new(search_term, where)
    return _.result # We will return a Fixnum here, or nil otherwise.
  end; self.instance_eval { alias [] find } # === FindLineInFile[]

end